function      Rays=Launch_Rays(Height,theta,phi,modelRfn,x_max)
%
%   Trace the ray paths from a specified launch point above a reference surface
%   USAGE:   Rays=Launch_Rays(Height,theta,phi,origin)
%
%   INPUTS:   Height      = height of ray bundle above TCS reference (x=0,y=0,z=0) (scalar)
%             theta       = incidence angle (rad. -ve downward)                    (1xNum_of_Rays)
%             phi         = azimuth angle   (rad. +ve from yz plane)               (1xNum_of_Rays)
%             modelRfn    = refractive index profile                               (structure)
%             x_max       = maximum ray extent 
%
%   OUTPUT:   Ray         1xNum_of_Rays structure
%                .launch  =[Height,theta,phi]
%                .above_surface = 0 if ray penetrated surface else 1
%                .n       = refractive index
%                .ds      = path length increment              (1xNumber_of_segments)
%                .xyz     = position in tcs coordinate system  (3xNumber_of_setments)
%                .tau     = ray tangent vector                 (3xNumber_of_setments)

%   METHOD: The program uses Equation 13, p. 124 from "Principles of Optics," Born and Wolf, Sixth Edition.
%   n*kappa*N = grad(n) - (grad(n) dot T)T    where,
%           n is index of refraction at sample point
%           kappa is the instantaneous curvature of the ray at the sample point
%           N is the principal normal of the ray path
%           grad(n) is the forward spatial gradient of the index of refraction
%           grad(n) dot T is the directional derivative of grad(n) in the direction
%                         of the ray at the sample point
%           T is the tangent vector at the sample point of the ray path
%
%   The program launches a ray from any 3-dimensional position in a right
%   handed, rectilinear coordinate system X-Y-Z, whose Z-axis is
%   co-linear with a surface normal and whose X-Y plane touches the
%   surface by tangency.  Also, the ray can be launched in any angle in
%   azimuth (phi) and elevation (theta).  Currently, for plotting
%   purposes, it's best to keep the ray within the octant -90.0 <= theta
%   <=0.0 and 0.0 <= phi <= 90.0
%
%   Computation terminates when Max_num_of_segments is reached or the ray penetrates
%       the surface (above_surface=1 when propagating; above_surface=0 when ray
%       has penetrated the ground)
%
%   Dennis Hancock
%   16Jan06
%   Adapted to function with WGS84 option CLR Feb-Mar 2006
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%These 2 parameters control accuracy and ray extent 
Max_num_of_segments = 100000;       %  Maximum number of calculated segments approximating ray path.
dl                  = 20.0;         %  Ray displacement increment along Tangent (in meters)

ray_x_position = zeros(1,Max_num_of_segments + 1);    % x value of ray segment along ray path
ray_y_position = zeros(1,Max_num_of_segments + 1);    % y value of ray segment along ray path
ray_z_position = zeros(1,Max_num_of_segments + 1);    % z value of ray segment along ray path

Tangent_x = zeros(1,Max_num_of_segments + 1);
Tangent_y = zeros(1,Max_num_of_segments + 1);
Tangent_z = zeros(1,Max_num_of_segments + 1);

ds          = zeros(1,Max_num_of_segments + 1);   %  Geometric path length of ray segment
dn          = zeros(1,Max_num_of_segments + 1);
s_error     = 0; %  Estimated error of straight line approximation to continuous ray path (in meters)

Tangent_x(1)     = 0;    %  Tangent vector at sample point on ray; norm(Tangent) = 1.0
Tangent_y(1)     = 0;
Tangent_z(1)     = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%% MAIN LOOP START  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define output structure
Num_of_Rays=length(theta);
Rays.launch     =[NaN,NaN,NaN];
Rays.above_surface=NaN;
Rays.dn         =NaN;
Rays.ds         =NaN;
Rays.xyz        =NaN*ones(3,1);
Rays.tau        =NaN*ones(3,1);
for ray=1:Num_of_Rays
    showprogress(ray)
    above_surface      = 1;
    inside_grid        = 1;
    i                  = 1;
    Tangent_x(1)            = cos(theta(ray))*sin(phi(ray));    % component of Tangent along ex
    Tangent_y(1)            = cos(theta(ray))*cos(phi(ray));    % component of Tangent along ey
    Tangent_z(1)            = sin(theta(ray));                  % component of Tangent along ez

    Position0             = [0 0 Height];         % starting position of source (in meters)
    ray_x_position(1) = Position0(1);             % First ray segment starts at source position
    ray_y_position(1) = Position0(2);
    ray_z_position(1) = Position0(3);

    ray_segment       = 0;  % number of ray segments computed before raypath loop terminates

    %  grad_answer = zeros(1,3);  % used for diagnostic purposes to check
    %  accuracy of Gradient_Of_Index_Of_Refraction calculation
    %%%%%%%%%%%%%%%%%%%%%%%%%  COMPUTE RAY PATH FOR RAY 'ray'  %%%%%%%%%%%%%%%%%%%
    while((ray_segment <= Max_num_of_segments) & (inside_grid) & ( above_surface))               % Compute while ray is above surface and loop count is within Max_num_of_segments
        i = i + 1;
        
        above_surface  = 1;
        inside_grid    = 1;
        h=Position0(3);
        if h < 0.0
            above_surface = 0;    % ray segment has passed through surface
        end
        if Position0(1)>x_max
            inside_grid = 0;
        end

        n         = Index_Of_Refraction(h,modelRfn);                     % get index of refraction at current sample point
        grad_n    = Gradient_Of_Index_Of_Refraction(Position0,modelRfn); % get forward gradient of index of refraction at current sample point
        %
        %    grad_answer = [grad_n(1)/grad_n_old(1) grad_n(2)/grad_n_old(2) grad_n(3)/grad_n_old(3)];   
        %
        directional_derivative_of_n = dot(grad_n, [Tangent_x(i-1), Tangent_y(i-1), Tangent_z(i-1)]); % directional derivative of grad_n with T
        P                           = 2*(grad_n - ...
            directional_derivative_of_n*[Tangent_x(i-1), Tangent_y(i-1), Tangent_z(i-1)])/n;  % P is in the plane formed by T and the component of grad_n perpendicular to T
        kappa                       = norm(P);
        %
        if(kappa > realmin('double'))  % propagate ray when there is enough of a gradient in the index of refraction at sample point

            radius_of_curvature         = 1.0/kappa;                                   % ray will turn on a circle whose radius
            %   is radius_of_curvature
            Principal_Normal            = radius_of_curvature*P;
            Center_of_Curvature         = Position0 + radius_of_curvature*Principal_Normal;
            L                           = Position0 + ...
                dl*[Tangent_x(i-1), Tangent_y(i-1), Tangent_z(i-1)];                   % a calculation aid--propagate ray from
            %   current sample point in direction of T
            rho                         = norm(L - Center_of_Curvature);               % distance from COC to new position along ray obtained
            % by projecting an amount dl along Tangent
            % from current sample point
            New_Position                = Center_of_Curvature + ...
                (radius_of_curvature/rho)*(L - Center_of_Curvature);
            rho                         = norm(New_Position - Position0);
            ds(i)                       = rho;                                         % update geometric path length
            s_error                     = s_error - rho^3/(24.*radius_of_curvature^2) ; % update approximate path length calculation
            dn(i)                       = n;
            %   due to using straight line approximation to
            %   a continuous arc
            Tangent_x(i)                  = (New_Position(1) - Position0(1))/rho;            % Calculate Tangent at new sample point at Position
            Tangent_y(i)                  = (New_Position(2) - Position0(2))/rho;
            Tangent_z(i)                  = (New_Position(3) - Position0(3))/rho;

        elseif i<=Max_num_of_segments
            % code to propagate ray in straight line when there is no component of refraction perpendicular to Tangent
            Tangent_x(i)                  =  Tangent_x(i-1);
            Tangent_y(i)                  =  Tangent_y(i-1);
            Tangent_z(i)                  =  Tangent_z(i-1);
            New_Position    = Position0 + ...
                dl*[Tangent_x(i), Tangent_y(i), Tangent_z(i)]; % propagate ray along original direction
            ds(i)           = dl;
            s_error         = s_error + 0.0;
            dn(i)           = n;
        end  %%%%%%%%%%%%%%  END OF COMPUTE RAY PATH FOR RAY 'ray'  %%%%%%%%%%%%%%%
        %
        Position0                       = New_Position;           % update new sample point position

        ray_x_position(i)           = New_Position(1);        % pick up ray segment endpoint coordinates for plotting purposes
        ray_y_position(i)           = New_Position(2);
        ray_z_position(i)           = New_Position(3);

        ray_segment                 = ray_segment + 1;   % increment ray segment counter
    end
    %
    Rays(ray).launch     =[Height,theta(ray),phi(ray)];
    Rays(ray).above_surface=above_surface;
    Rays(ray).dn=dn(1:ray_segment);
    Rays(ray).ds=ds(1:ray_segment);
    Rays(ray).xyz=[ray_x_position(1:ray_segment); ray_y_position(1:ray_segment); ray_z_position(1:ray_segment)];
    Rays(ray).tau=[Tangent_x(1:ray_segment); Tangent_y(1:ray_segment); Tangent_z(1:ray_segment)];
end  %%%%%%%%%%%%%%%%%%%%    END OF MAIN LOOP  %%%%%%%%%%%%%%%%%%%%%%%%%%%%
return